@@ -2,6 +2,7 @@
 
 #ifdef HAPPY_GHC
 #define ILIT(n) n#
+#define FAST_INT_BINDING(n) !(n)
 #define IBOX(n) (Happy_GHC_Exts.I# (n))
 #define FAST_INT Happy_GHC_Exts.Int#
 #define LT(n,m) (n Happy_GHC_Exts.<# m)
@@ -14,6 +15,7 @@
 #define IF_GHC(x) (x)
 #else
 #define ILIT(n) (n)
+#define FAST_INT_BINDING(n) (n)
 #define IBOX(n) (n)
 #define FAST_INT Int
 #define LT(n,m) (n < m)
@@ -107,14 +109,15 @@ happyDoAction i tk st
 						 ++ show IBOX(new_state)
 						 ++ "\n")
 				     happyShift new_state i tk st
-				     where new_state = MINUS(n,(ILIT(1) :: FAST_INT))
-   where off    = indexShortOffAddr happyActOffsets st
-	 off_i  = PLUS(off,i)
+				     where FAST_INT_BINDING(new_state) = MINUS(n,(ILIT(1) :: FAST_INT))
+   where FAST_INT_BINDING(off)    = indexShortOffAddr happyActOffsets st
+         FAST_INT_BINDING(off_i)  = PLUS(off,i)
 	 check  = if GTE(off_i,(ILIT(0) :: FAST_INT))
 			then EQ(indexShortOffAddr happyCheck off_i, i)
 			else False
- 	 action | check     = indexShortOffAddr happyTable off_i
-		| otherwise = indexShortOffAddr happyDefActions st
+         FAST_INT_BINDING(action)
+          | check     = indexShortOffAddr happyTable off_i
+          | otherwise = indexShortOffAddr happyDefActions st
 
 #ifdef HAPPY_GHC
 #undef __GLASGOW_HASKELL__
@@ -127,22 +130,12 @@ happyDoAction i tk st
 
 #ifdef HAPPY_GHC
 indexShortOffAddr (HappyA# arr) off =
-HAPPY_IF_GHC_GT_500
 	Happy_GHC_Exts.narrow16Int# i
-HAPPY_ELIF_GHC_500
-	Happy_GHC_Exts.intToInt16# i
-HAPPY_ELSE
-	Happy_GHC_Exts.iShiftRA# (Happy_GHC_Exts.iShiftL# i 16#) 16#
-HAPPY_ENDIF
   where
-HAPPY_IF_GHC_GE_503
-	i = Happy_GHC_Exts.word2Int# (Happy_GHC_Exts.or# (Happy_GHC_Exts.uncheckedShiftL# high 8#) low)
-HAPPY_ELSE
-	i = Happy_GHC_Exts.word2Int# (Happy_GHC_Exts.or# (Happy_GHC_Exts.shiftL# high 8#) low)
-HAPPY_ENDIF
-	high = Happy_GHC_Exts.int2Word# (Happy_GHC_Exts.ord# (Happy_GHC_Exts.indexCharOffAddr# arr (off' Happy_GHC_Exts.+# 1#)))
-	low  = Happy_GHC_Exts.int2Word# (Happy_GHC_Exts.ord# (Happy_GHC_Exts.indexCharOffAddr# arr off'))
-	off' = off Happy_GHC_Exts.*# 2#
+	!i = Happy_GHC_Exts.word2Int# (Happy_GHC_Exts.or# (Happy_GHC_Exts.uncheckedShiftL# high 8#) low)
+	!high = Happy_GHC_Exts.int2Word# (Happy_GHC_Exts.ord# (Happy_GHC_Exts.indexCharOffAddr# arr (off' Happy_GHC_Exts.+# 1#)))
+	!low  = Happy_GHC_Exts.int2Word# (Happy_GHC_Exts.ord# (Happy_GHC_Exts.indexCharOffAddr# arr off'))
+	!off' = off Happy_GHC_Exts.*# 2#
 #else
 indexShortOffAddr arr off = arr Happy_Data_Array.! off
 #endif
@@ -172,7 +165,7 @@ newtype HappyState b c = HappyState
 -- Shifting a token
 
 happyShift new_state ERROR_TOK tk st sts stk@(x `HappyStk` _) =
-     let i = GET_ERROR_TOKEN(x) in
+     let FAST_INT_BINDING(i) = GET_ERROR_TOKEN(x) in
 --     trace "shifting the error token" $
      DO_ACTION(new_state,i,tk,CONS(st,sts),stk)
 
@@ -216,19 +209,19 @@ happyMonadReduce k nt fn ERROR_TOK tk st sts stk
      = happyFail ERROR_TOK tk st sts stk
 happyMonadReduce k nt fn j tk st sts stk =
         happyThen1 (fn stk tk) (\r -> GOTO(action) nt j tk st1 sts1 (r `HappyStk` drop_stk))
-       where sts1@(CONS(st1@HAPPYSTATE(action),_)) = happyDrop k CONS(st,sts)
+       where FAST_INT_BINDING(sts1@(CONS(st1@HAPPYSTATE(action),_))) = happyDrop k CONS(st,sts)
              drop_stk = happyDropStk k stk
 
 happyMonad2Reduce k nt fn ERROR_TOK tk st sts stk
      = happyFail ERROR_TOK tk st sts stk
 happyMonad2Reduce k nt fn j tk st sts stk =
        happyThen1 (fn stk tk) (\r -> happyNewToken new_state sts1 (r `HappyStk` drop_stk))
-       where sts1@(CONS(st1@HAPPYSTATE(action),_)) = happyDrop k CONS(st,sts)
+       where FAST_INT_BINDING(sts1@(CONS(st1@HAPPYSTATE(action),_))) = happyDrop k CONS(st,sts)
              drop_stk = happyDropStk k stk
 #if defined(HAPPY_ARRAY)
-             off    = indexShortOffAddr happyGotoOffsets st1
-             off_i  = PLUS(off,nt)
-             new_state = indexShortOffAddr happyTable off_i
+             FAST_INT_BINDING(off) = indexShortOffAddr happyGotoOffsets st1
+             FAST_INT_BINDING(off_i) = PLUS(off,nt)
+             FAST_INT_BINDING(new_state) = indexShortOffAddr happyTable off_i
 #else
              new_state = action
 #endif
@@ -246,9 +239,9 @@ happyDropStk n (x `HappyStk` xs) = happyDropStk MINUS(n,(ILIT(1)::FAST_INT)) xs
 happyGoto nt j tk st = 
    DEBUG_TRACE(", goto state " ++ show IBOX(new_state) ++ "\n")
    happyDoAction j tk new_state
-   where off    = indexShortOffAddr happyGotoOffsets st
-	 off_i  = PLUS(off,nt)
- 	 new_state = indexShortOffAddr happyTable off_i
+   where FAST_INT_BINDING(off) = indexShortOffAddr happyGotoOffsets st
+         FAST_INT_BINDING(off_i) = PLUS(off,nt)
+         FAST_INT_BINDING(new_state) = indexShortOffAddr happyTable off_i
 #else
 happyGoto action j tk st = action j j tk (HappyState action)
 #endif
